#!/usr/bin/python

from twisted.application import internet,service
from twisted.internet.protocol import ServerFactory
from twisted.python import log
from twisted.application.service import IProcess

#import logging

import sys

sys.path.append("/opt/ardi/srv/consolidator")

import time
import incoming
import subscription
import threading
import argparse
import logging
import requests
import xmltodict
import traceback

class Core:
    def __init__(self):
        self.points = {}
        self.pointimes = {}
        self.lock = threading.Lock()
        self.lastupdate = 0
        self.subscriptions = []
        self.maxusers = 0
        pass

    def FetchServerInfo(self,srv):
        success = False
        #while success == False:
        try:
            print("Checking http://" + srv + "/api/connect for server details...")
            r = requests.get('http://' + srv + "/api/connect")
            parts = xmltodict.parse(r.text)
            try:
                for bt in parts['ardi']['setting']:
                    if bt['@name'] == 'users':
                        if bt['#text'] != 'Unlimited':
                            try:
                                self.maxusers = int(bt['#text'])
                            except:
                                pass
            except:
                pass
            success = True
        except:
            traceback.print_exc()
            #time.sleep(10)
            pass

        logging.info('Connected to Server ' + str(srv) + " - " +str(self.maxusers) + " Concurrent User Max")
        print('Connected to Server ' + str(srv) + " - " +str(self.maxusers) + " Concurrent User Max")
        #print "Server: " + str(srv)
    
    def NewDataArrived(self, name, value):
        self.lock.acquire()
        value = value.strip()
        try:
            if self.points[name] != value:
                self.points[name] = value
                self.pointimes[name] = time.time()
        except:
            self.points[name] = value
            self.pointimes[name] = time.time()
            
        self.lock.release()
        self.lastupdate = time.time()

class IncomingService(service.Service):
    def __init__(self, core):
        self.core = core
        pass
    
    def startService(self):
        service.Service.startService(self)
        pass

class OutgoingService(service.Service):
    def __init__(self,core):
        self.core = core
        pass
    
    def startService(self):
        service.Service.startService(self)
        
class IncomingFactory(ServerFactory):
    protocol = incoming.Submission
    def __init__(self, service):
        self.service = service
        
    def buildProtocol(self, addr):
        return incoming.Submission(self)

#class ConsolidatorFactory:
def ConsolidatorFactory(inport,outport,srv):
    
    core = Core()
    
    core.FetchServerInfo(srv)
    
    top_service = service.MultiService()

    inserv = IncomingService(core)
    inserv.setServiceParent(top_service)

    infact = IncomingFactory(inserv)
    tcp_service = internet.TCPServer(int(inport),infact)
    tcp_service.setServiceParent(top_service)

    site = subscription.GetService(core)
    tcp_service = internet.TCPServer(int(outport),site)
    tcp_service.setServiceParent(top_service)

    return top_service
        